local building = {}
building.__index = building
local sideSlices,bottomSlices
sideSlices = {}
bottomSlices = {}
for i=0,30 do
  table.insert(sideSlices, love.graphics.newQuad(31-i,0,2,160,32,160))
  table.insert(bottomSlices, love.graphics.newQuad(0,i,160,2,160,32))
end

local function new(x,y,width,height,floors,leftSide,rightSide,front,roof)
  return setmetatable({
    x = x,
    y = y,
    width = width,
    height = height,
    floors = floors,
    leftBatch = love.graphics.newSpriteBatch(leftSide, floors*32*6),
    rightBatch = love.graphics.newSpriteBatch(rightSide, floors*32*6),
    frontBatch =  love.graphics.newSpriteBatch(front, floors*32*6),
    backBatch = love.graphics.newSpriteBatch(front, floors*32*6),
    roofQuad = love.graphics.newQuad(0,0,width,height,16,16),
    roofCoords = {0,0},
    roofImage = roof
  },building)
end

function building:update(playerX,playerY)
  local distance = 100
  local edges = {}
  local growth = 0.005
  local topLeft,topRight,bottomLeft,bottomRight = {},{},{},{}
  for i=0,30*self.floors do
    topLeft[i+1] = {
      px = math.floor((distance * (self.x - playerX)) / (distance + -i*growth*distance) + playerX),
      py = math.floor((distance * (self.y - playerY)) / (distance + -i*growth*distance) + playerY),
    }
    topRight[i+1] = {
      px = math.floor((distance * (self.x + self.width - playerX)) / (distance + -i*growth*distance) + playerX),
      py = math.floor((distance * (self.y - playerY)) / (distance + -i*growth*distance) + playerY),
    }
    bottomRight[i+1] = {
      px = math.floor((distance * (self.x + self.width - playerX)) / (distance  + -i*growth*distance) + playerX),
      py = math.floor((distance * (self.y + self.height - playerY)) / (distance + -i*growth*distance) + playerY),
    }
    bottomLeft[i+1] = {
      px = math.floor((distance * (self.x - playerX)) / (distance  + -i*growth*distance) + playerX),
      py = math.floor((distance * (self.y + self.height - playerY)) / (distance + -i*growth*distance) + playerY),
    }
  end
  local finalTL, finalBr = topLeft[(self.floors*30)+1], bottomRight[(self.floors*30)+1]
  self.roofQuad:setViewport(0,0, math.abs(finalTL.px - finalBr.px), math.abs(finalTL.py - finalBr.py))
  self.roofCoords = {finalTL.px+1,finalTL.py+1}
  -- self:updateLeft(topLeft,bottomLeft)
  -- self:updateBatch(topRight,bottomRight)
  -- self:updateBack(topLeft,topRight)
  -- self:updateFront(bottomLeft,bottomRight)
  self:updateBatch(self.leftBatch,sideSlices,topLeft,bottomLeft)
  self:updateBatch(self.rightBatch,sideSlices,topRight,bottomRight)
  self:updateBatch(self.backBatch,bottomSlices,topLeft,topRight)
  self:updateBatch(self.frontBatch,bottomSlices,bottomLeft,bottomRight)
end

function building:updateBatch(batch,slices,drawCoords,scaleCoords)
  batch:clear()
  for i,drawCoord in ipairs(drawCoords) do
    local scale = math.max(math.abs(drawCoord.py - scaleCoords[i].py),math.abs(drawCoord.px - scaleCoords[i].px))/160
    batch:addq(slices[(i%31)+1], drawCoord.px, drawCoord.py,0,scale)
  end
end

function building:draw(playerX,playerY)
  love.graphics.draw(self.backBatch)
  if playerY < self.y + self.height then
    love.graphics.draw(self.frontBatch)
    if playerY < self.y then
      if playerX > self.x + self.width then
        love.graphics.draw(self.leftBatch)
        love.graphics.draw(self.rightBatch)
      else
        love.graphics.draw(self.rightBatch)
        love.graphics.draw(self.leftBatch)
      end
      love.graphics.draw(self.backBatch)
    else
      love.graphics.draw(self.backBatch)
      if playerX > self.x + self.width then
        love.graphics.draw(self.leftBatch)
        love.graphics.draw(self.rightBatch)
      else
        love.graphics.draw(self.rightBatch)
        love.graphics.draw(self.leftBatch)
      end
    end
  else
    love.graphics.draw(self.backBatch)
    if playerX > self.x + self.width then
      love.graphics.draw(self.leftBatch)
      love.graphics.draw(self.rightBatch)
    else
      love.graphics.draw(self.rightBatch)
      love.graphics.draw(self.leftBatch)
    end
    love.graphics.draw(self.frontBatch)
  end
  love.graphics.drawq(self.roofImage, self.roofQuad, unpack(self.roofCoords))
end

-- the module
return setmetatable({new = new, isvector = isvector},
{__call = function(_, ...) return new(...) end})