Camera = require('camera')
Building = require('building')

local vector = require 'vector'
local drawableBuildings
local function buildBuilding(x,y,width,height,floors)
  local newBuilding = Building.new(x,y,width,height,floors,leftImage,rightImage,bottomImage,roofImage)
  table.insert(buildings, newBuilding)
end

function love.load()
  roofImage = love.graphics.newImage('roof.png')
  roofImage:setWrap('repeat','repeat')
  leftImage = love.graphics.newImage('left.png')
  leftImage:setWrap('repeat','repeat')
  leftImage:setFilter('nearest','nearest')
  rightImage = love.graphics.newImage('right.png')
  rightImage:setWrap('repeat','repeat')
  rightImage:setFilter('nearest','nearest')
  bottomImage = love.graphics.newImage('bottom.png')
  bottomImage:setFilter('nearest','nearest')
  playerX, playerY = 200,200
  cam = Camera(300,100)
love.graphics.setDefaultImageFilter('nearest','nearest')
  local buildingLocations = {}--,{0,300,3},{300,300,3},{600,0,5},{600,600,2}}--{0,0,7}{0,0,7}{0,0,7}{0,0,7}{0,0,7}{0,0,7}}--,{200,0,2},{400,0,2}}
  for i=0,20 do
    for y=0,20 do
      table.insert(buildingLocations, {i*320,y*320, math.random(160,256), math.random(160,256),math.random(1,4)})
    end
  end
  buildings = {}
  for i, location in ipairs(buildingLocations) do
    if location then
      buildBuilding(unpack(location))
    end
  end
  drawableBuildings = {}
end

function love.draw()
  cam:draw(draw_walls)
end

local function updateDrawable()
  drawableBuildings = {}
  local cx,cy
  for i,building in ipairs(buildings) do
    cx,cy = cam:cameraCoords(building.x,building.y)
    if cx < -building.width or cx > love.graphics.getWidth() + (building.width) or cy < -building.height or cy > love.graphics.getHeight() + (building.height) then
      -- break
    else
      table.insert(drawableBuildings, building)
    end
    
  end
end

function draw_walls()
  table.sort(drawableBuildings,function(a,b)
    local dxA,dyA,dxB,dyB,dtx,dmx,dbx,dty,dmy,dby, distA, distB
    dtx = math.abs(a.x - playerX)
    dbx = math.abs(a.x + (a.width) - playerX)
    dmx = math.abs(a.x + (a.width/2) - playerX)
    dty = math.abs(a.y - playerY)
    dmy = math.abs(a.y + (a.height) - playerY)
    dby = math.abs(a.y + (a.height/2) - playerY)
    dxA = math.min(math.min(dtx,dbx),dmx)
    dyA = math.min(math.min(dty,dby),dmy)
    distA = math.sqrt(dxA*dxA + dyA*dyA)

    dtx = math.abs(b.x - playerX)
    dbx = math.abs(b.x + (b.width) - playerX)
    dmx = math.abs(b.x + (b.width/2) - playerX)
    dty = math.abs(b.y - playerY)
    dmy = math.abs(b.y + (b.height) - playerY)
    dby = math.abs(b.y + (b.height/2) - playerY)
    dxB = math.min(math.min(dtx,dmx),dbx)
    dyB = math.min(math.min(dty,dmy),dby)
    distB = math.sqrt(dxB*dxB + dyB*dyB)
    return distA > distB
  end)
  
  for i,building in ipairs(drawableBuildings) do
    building:draw(playerX,playerY)
  end

  love.graphics.circle('fill',playerX,playerY,2)
end

function love.update(dt)
  if love.keyboard.isDown('up') then
    playerY = playerY - 5
  elseif love.keyboard.isDown('down') then
    playerY = playerY + 5
  end
  if love.keyboard.isDown('left') then
    playerX = playerX - 5
  elseif love.keyboard.isDown('right') then
    playerX = playerX + 5
  end
  if love.keyboard.isDown('escape') then
    love.event.quit()
  end
  updateDrawable()
  for i,building in ipairs(drawableBuildings) do
    building:update(playerX,playerY)
  end
  cam:lookAt(playerX,playerY)
end
